@@ -12,8 +12,7 @@ import stat
 from snakemake.logging import logger
 from snakemake.jobs import Job
 from snakemake.exceptions import WorkflowError
-from snakemake.caching.hash import ProvenanceHashMap
-from snakemake.caching import LOCATION_ENVVAR, AbstractOutputFileCache
+from snakemake.caching import AbstractOutputFileCache
 
 
 class OutputFileCache(AbstractOutputFileCache):
@@ -66,11 +65,11 @@ class OutputFileCache(AbstractOutputFileCache):
             ):
                 if not os.path.exists(outputfile):
                     raise WorkflowError(
-                        "Cannot move output file {} to cache. It does not exist "
+                        f"Cannot move output file {outputfile} to cache. It does not exist "
                         "(maybe it was not created by the job?)."
                     )
                 self.check_writeable(cachefile)
-                logger.info("Moving output file {} to cache.".format(outputfile))
+                logger.info(f"Moving output file {outputfile} to cache.")
 
                 tmp = tmpdir / cachefile.name
                 # First move is performed into a tempdir (it might involve a copy if not on the same FS).
@@ -95,7 +94,6 @@ class OutputFileCache(AbstractOutputFileCache):
         for outputfile, cachefile in self.get_outputfiles_and_cachefiles(
             job, cache_mode
         ):
-
             if not cachefile.exists():
                 self.raise_cache_miss_exception(job)
 
@@ -123,7 +121,6 @@ class OutputFileCache(AbstractOutputFileCache):
         for outputfile, cachefile in self.get_outputfiles_and_cachefiles(
             job, cache_mode
         ):
-
             if not cachefile.exists():
                 return False
 
@@ -147,7 +144,7 @@ class OutputFileCache(AbstractOutputFileCache):
 
     def symlink(self, path, outputfile, utime=True):
         if os.utime in os.supports_follow_symlinks or not utime:
-            logger.info("Symlinking output file {} from cache.".format(outputfile))
+            logger.info(f"Symlinking output file {outputfile} from cache.")
             os.symlink(path, outputfile)
             if utime:
                 os.utime(outputfile, follow_symlinks=False)
