@@ -3,60 +3,57 @@ __copyright__ = "Copyright 2022, Johannes Köster"
 __email__ = "johannes.koester@uni-due.de"
 __license__ = "MIT"
 
-from abc import abstractmethod
+from abc import ABC, abstractmethod
 import asyncio
 import os
 import sys
 import contextlib
 import time
-import datetime
 import json
-import textwrap
 import stat
 import shutil
 import shlex
 import threading
 import concurrent.futures
 import subprocess
-import signal
 import tempfile
 from functools import partial
-from itertools import chain
 from collections import namedtuple
-import random
 import base64
+from typing import List
 import uuid
 import re
 import math
+from snakemake.interfaces import (
+    DAGExecutorInterface,
+    ExecutorJobInterface,
+    GroupJobExecutorInterface,
+    SingleJobExecutorInterface,
+    WorkflowExecutorInterface,
+)
 from snakemake.target_jobs import encode_target_jobs_cli_args
 from fractions import Fraction
 
-from snakemake.jobs import Job
 from snakemake.shell import shell
 from snakemake.logging import logger
 from snakemake.stats import Stats
-from snakemake.utils import format, Unformattable, makedirs
+from snakemake.utils import makedirs
 from snakemake.io import get_wildcard_names, Wildcards
 from snakemake.exceptions import print_exception, get_exception_origin
 from snakemake.exceptions import format_error, RuleException, log_verbose_traceback
 from snakemake.exceptions import (
-    ProtectedOutputException,
     WorkflowError,
-    ImproperShadowException,
     SpawnedJobError,
     CacheMissException,
 )
 from snakemake.common import (
     Mode,
-    __version__,
     get_container_image,
     get_uuid,
     lazy_property,
-    dict_to_key_value_args,
     async_lock,
 )
-from snakemake.executors.common import format_cli_arg, format_cli_pos_arg, join_cli_args
-from snakemake.io import _IOFile
+from snakemake.executors.common import format_cli_arg, join_cli_args
 
 
 # TODO move each executor into a separate submodule
@@ -70,11 +67,11 @@ async def sleep():
         await asyncio.sleep(1)
 
 
-class AbstractExecutor:
+class AbstractExecutor(ABC):
     def __init__(
         self,
-        workflow,
-        dag,
+        workflow: WorkflowExecutorInterface,
+        dag: DAGExecutorInterface,
         printreason=False,
         quiet=False,
         printshellcmds=False,
@@ -118,7 +115,7 @@ class AbstractExecutor:
             "--set-resource-scopes", self.workflow.overwrite_resource_scopes
         )
 
-    def get_resource_declarations_dict(self, job):
+    def get_resource_declarations_dict(self, job: ExecutorJobInterface):
         def isdigit(i):
             s = str(i)
             # Adapted from https://stackoverflow.com/a/1265696
@@ -137,14 +134,20 @@ class AbstractExecutor:
             and isdigit(value) and resource not in excluded_resources
         }
 
-    def get_resource_declarations(self, job):
+    def get_resource_declarations(self, job: ExecutorJobInterface):
         resources = [
             f"{resource}={value}"
             for resource, value in self.get_resource_declarations_dict(job).items()
         ]
         return format_cli_arg("--resources", resources)
 
-    def run_jobs(self, jobs, callback=None, submit_callback=None, error_callback=None):
+    def run_jobs(
+        self,
+        jobs: List[ExecutorJobInterface],
+        callback=None,
+        submit_callback=None,
+        error_callback=None,
+    ):
         """Run a list of jobs that is ready at a given point in time.
 
         By default, this method just runs each job individually.
@@ -159,7 +162,13 @@ class AbstractExecutor:
                 error_callback=error_callback,
             )
 
-    def run(self, job, callback=None, submit_callback=None, error_callback=None):
+    def run(
+        self,
+        job: ExecutorJobInterface,
+        callback=None,
+        submit_callback=None,
+        error_callback=None,
+    ):
         """Run a specific job or group job."""
         self._run(job)
         callback(job)
@@ -172,28 +181,30 @@ class AbstractExecutor:
     def cancel(self):
         ...
 
-    def _run(self, job):
+    def _run(self, job: ExecutorJobInterface):
         job.check_protected_output()
         self.printjob(job)
 
-    def rule_prefix(self, job):
+    def rule_prefix(self, job: ExecutorJobInterface):
         return "local " if job.is_local else ""
 
-    def printjob(self, job):
+    def printjob(self, job: ExecutorJobInterface):
         job.log_info(skip_dynamic=True)
 
-    def print_job_error(self, job, msg=None, **kwargs):
+    def print_job_error(self, job: ExecutorJobInterface, msg=None, **kwargs):
         job.log_error(msg, **kwargs)
 
-    def handle_job_success(self, job):
-        pass
+    @abstractmethod
+    def handle_job_success(self, job: ExecutorJobInterface):
+        ...
 
-    def handle_job_error(self, job):
-        pass
+    @abstractmethod
+    def handle_job_error(self, job: ExecutorJobInterface):
+        ...
 
 
 class DryrunExecutor(AbstractExecutor):
-    def printjob(self, job):
+    def printjob(self, job: ExecutorJobInterface):
         super().printjob(job)
         if job.is_group():
             for j in job.jobs:
@@ -201,7 +212,7 @@ class DryrunExecutor(AbstractExecutor):
         else:
             self.printcache(job)
 
-    def printcache(self, job):
+    def printcache(self, job: ExecutorJobInterface):
         cache_mode = self.workflow.get_cache_mode(job.rule)
         if cache_mode:
             if self.workflow.output_file_cache.exists(job, cache_mode):
@@ -217,12 +228,24 @@ class DryrunExecutor(AbstractExecutor):
                     )
                 )
 
+    def cancel(self):
+        pass
+
+    def shutdown(self):
+        pass
+
+    def handle_job_success(self, job: ExecutorJobInterface):
+        pass
+
+    def handle_job_error(self, job: ExecutorJobInterface):
+        pass
+
 
 class RealExecutor(AbstractExecutor):
     def __init__(
         self,
-        workflow,
-        dag,
+        workflow: WorkflowExecutorInterface,
+        dag: DAGExecutorInterface,
         printreason=False,
         quiet=False,
         printshellcmds=False,
@@ -241,10 +264,10 @@ class RealExecutor(AbstractExecutor):
         self.stats = Stats()
         self.snakefile = workflow.main_snakefile
 
-    def register_job(self, job):
+    def register_job(self, job: ExecutorJobInterface):
         job.register()
 
-    def _run(self, job, callback=None, error_callback=None):
+    def _run(self, job: ExecutorJobInterface, callback=None, error_callback=None):
         super()._run(job)
         self.stats.report_job_start(job)
 
@@ -261,7 +284,7 @@ class RealExecutor(AbstractExecutor):
 
     def handle_job_success(
         self,
-        job,
+        job: ExecutorJobInterface,
         upload_remote=True,
         handle_log=True,
         handle_touch=True,
@@ -278,7 +301,7 @@ class RealExecutor(AbstractExecutor):
         )
         self.stats.report_job_end(job)
 
-    def handle_job_error(self, job, upload_remote=True):
+    def handle_job_error(self, job: ExecutorJobInterface, upload_remote=True):
         job.postprocess(
             error=True,
             assume_shared_fs=self.assume_shared_fs,
@@ -374,12 +397,11 @@ class RealExecutor(AbstractExecutor):
     def get_workdir_arg(self):
         return self.workflow_property_to_arg("overwrite_workdir", flag="--directory")
 
-    def get_job_args(self, job, **kwargs):
+    def get_job_args(self, job: ExecutorJobInterface, **kwargs):
         return join_cli_args(
             [
                 format_cli_arg(
-                    "--target-jobs",
-                    encode_target_jobs_cli_args(job.get_target_spec()),
+                    "--target-jobs", encode_target_jobs_cli_args(job.get_target_spec())
                 ),
                 # Restrict considered rules for faster DAG computation.
                 # This does not work for updated jobs because they need
@@ -414,17 +436,16 @@ class RealExecutor(AbstractExecutor):
     def get_exec_mode(self):
         ...
 
-    @abstractmethod
     def get_envvar_declarations(self):
-        ...
-
-    def get_job_exec_prefix(self, job):
         return ""
 
-    def get_job_exec_suffix(self, job):
+    def get_job_exec_prefix(self, job: ExecutorJobInterface):
         return ""
 
-    def format_job_exec(self, job):
+    def get_job_exec_suffix(self, job: ExecutorJobInterface):
+        return ""
+
+    def format_job_exec(self, job: ExecutorJobInterface):
         prefix = self.get_job_exec_prefix(job)
         if prefix:
             prefix += " &&"
@@ -446,7 +467,13 @@ class RealExecutor(AbstractExecutor):
 
 
 class TouchExecutor(RealExecutor):
-    def run(self, job, callback=None, submit_callback=None, error_callback=None):
+    def run(
+        self,
+        job: ExecutorJobInterface,
+        callback=None,
+        submit_callback=None,
+        error_callback=None,
+    ):
         super()._run(job)
         try:
             # Touching of output files will be done by handle_job_success
@@ -456,9 +483,21 @@ class TouchExecutor(RealExecutor):
             print_exception(ex, self.workflow.linemaps)
             error_callback(job)
 
-    def handle_job_success(self, job):
+    def handle_job_success(self, job: ExecutorJobInterface):
         super().handle_job_success(job, ignore_missing_output=True)
 
+    def cancel(self):
+        pass
+
+    def shutdown(self):
+        pass
+
+    def get_exec_mode(self):
+        raise NotImplementedError()
+
+    def get_python_executable(self):
+        raise NotImplementedError()
+
 
 _ProcessPoolExceptions = (KeyboardInterrupt,)
 try:
@@ -472,8 +511,8 @@ except ImportError:
 class CPUExecutor(RealExecutor):
     def __init__(
         self,
-        workflow,
-        dag,
+        workflow: WorkflowExecutorInterface,
+        dag: DAGExecutorInterface,
         workers,
         printreason=False,
         quiet=False,
@@ -496,15 +535,16 @@ class CPUExecutor(RealExecutor):
 
         # Zero thread jobs do not need a thread, but they occupy additional workers.
         # Hence we need to reserve additional workers for them.
-        self.workers = workers + 5
+        workers = workers + 5 if workers is not None else 5
+        self.workers = workers
         self.pool = concurrent.futures.ThreadPoolExecutor(max_workers=self.workers)
 
     @property
     def job_specific_local_groupid(self):
         return False
 
-    def get_job_exec_prefix(self, job):
-        return f"cd {self.workflow.workdir_init}"
+    def get_job_exec_prefix(self, job: ExecutorJobInterface):
+        return f"cd {shlex.quote(self.workflow.workdir_init)}"
 
     def get_exec_mode(self):
         return Mode.subprocess
@@ -515,10 +555,16 @@ class CPUExecutor(RealExecutor):
     def get_envvar_declarations(self):
         return ""
 
-    def get_job_args(self, job, **kwargs):
+    def get_job_args(self, job: ExecutorJobInterface, **kwargs):
         return f"{super().get_job_args(job, **kwargs)} --quiet"
 
-    def run(self, job, callback=None, submit_callback=None, error_callback=None):
+    def run(
+        self,
+        job: ExecutorJobInterface,
+        callback=None,
+        submit_callback=None,
+        error_callback=None,
+    ):
         super()._run(job)
 
         if job.is_group():
@@ -537,7 +583,7 @@ class CPUExecutor(RealExecutor):
 
         future.add_done_callback(partial(self._callback, job, callback, error_callback))
 
-    def job_args_and_prepare(self, job):
+    def job_args_and_prepare(self, job: ExecutorJobInterface):
         job.prepare()
 
         conda_env = (
@@ -579,7 +625,7 @@ class CPUExecutor(RealExecutor):
             self.workflow.sourcecache.runtime_cache_path,
         )
 
-    def run_single_job(self, job):
+    def run_single_job(self, job: SingleJobExecutorInterface):
         if (
             self.use_threads
             or (not job.is_shadow and not job.is_run)
@@ -593,7 +639,7 @@ class CPUExecutor(RealExecutor):
             future = self.pool.submit(self.cached_or_run, job, self.spawn_job, job)
         return future
 
-    def run_group_job(self, job):
+    def run_group_job(self, job: GroupJobExecutorInterface):
         """Run a pipe or service group job.
 
         This lets all items run simultaneously."""
@@ -632,14 +678,14 @@ class CPUExecutor(RealExecutor):
                 return
             time.sleep(1)
 
-    def spawn_job(self, job):
+    def spawn_job(self, job: SingleJobExecutorInterface):
         cmd = self.format_job_exec(job)
         try:
             subprocess.check_call(cmd, shell=True)
         except subprocess.CalledProcessError as e:
             raise SpawnedJobError()
 
-    def cached_or_run(self, job, run_func, *args):
+    def cached_or_run(self, job: SingleJobExecutorInterface, run_func, *args):
         """
         Either retrieve result from cache, or run job with given function.
         """
@@ -660,7 +706,9 @@ class CPUExecutor(RealExecutor):
     def cancel(self):
         self.pool.shutdown()
 
-    def _callback(self, job, callback, error_callback, future):
+    def _callback(
+        self, job: SingleJobExecutorInterface, callback, error_callback, future
+    ):
         try:
             ex = future.exception()
             if ex is not None:
@@ -674,14 +722,14 @@ class CPUExecutor(RealExecutor):
             error_callback(job)
         except (Exception, BaseException) as ex:
             self.print_job_error(job)
-            if self.workflow.verbose or (job.is_run and not job.is_group()):
+            if self.workflow.verbose or (not job.is_group() and not job.is_shell):
                 print_exception(ex, self.workflow.linemaps)
             error_callback(job)
 
-    def handle_job_success(self, job):
+    def handle_job_success(self, job: ExecutorJobInterface):
         super().handle_job_success(job)
 
-    def handle_job_error(self, job):
+    def handle_job_error(self, job: ExecutorJobInterface):
         super().handle_job_error(job)
         if not self.keepincomplete:
             job.cleanup()
@@ -699,8 +747,8 @@ class ClusterExecutor(RealExecutor):
 
     def __init__(
         self,
-        workflow,
-        dag,
+        workflow: WorkflowExecutorInterface,
+        dag: DAGExecutorInterface,
         cores,
         jobname="snakejob.{name}.{jobid}.sh",
         printreason=False,
@@ -807,7 +855,7 @@ class ClusterExecutor(RealExecutor):
     def get_exec_mode(self):
         return Mode.cluster
 
-    def get_job_args(self, job):
+    def get_job_args(self, job: ExecutorJobInterface):
         waitfiles_parameter = ""
         if self.assume_shared_fs:
             wait_for_files = []
@@ -854,7 +902,7 @@ class ClusterExecutor(RealExecutor):
     def cancel(self):
         self.shutdown()
 
-    def _run(self, job, callback=None, error_callback=None):
+    def _run(self, job: ExecutorJobInterface, callback=None, error_callback=None):
         if self.assume_shared_fs:
             job.remove_existing_output()
             job.download_remote_input()
@@ -866,10 +914,10 @@ class ClusterExecutor(RealExecutor):
             self._tmpdir = tempfile.mkdtemp(dir=".snakemake", prefix="tmp.")
         return os.path.abspath(self._tmpdir)
 
-    def get_jobname(self, job):
+    def get_jobname(self, job: ExecutorJobInterface):
         return job.format_wildcards(self.jobname, cluster=self.cluster_wildcards(job))
 
-    def get_jobscript(self, job):
+    def get_jobscript(self, job: ExecutorJobInterface):
         f = self.get_jobname(job)
 
         if os.path.sep in f:
@@ -880,7 +928,7 @@ class ClusterExecutor(RealExecutor):
 
         return os.path.join(self.tmpdir, f)
 
-    def write_jobscript(self, job, jobscript):
+    def write_jobscript(self, job: ExecutorJobInterface, jobscript):
         exec_job = self.format_job_exec(job)
 
         try:
@@ -897,12 +945,12 @@ class ClusterExecutor(RealExecutor):
                     "Make sure that your custom jobscript is defined as expected."
                 )
 
-        logger.debug("Jobscript:\n{}".format(content))
+        logger.debug(f"Jobscript:\n{content}")
         with open(jobscript, "w") as f:
             print(content, file=f)
         os.chmod(jobscript, os.stat(jobscript).st_mode | stat.S_IXUSR | stat.S_IRUSR)
 
-    def cluster_params(self, job):
+    def cluster_params(self, job: ExecutorJobInterface):
         """Return wildcards object for job from cluster_config."""
         cluster = self.cluster_config.get("__default__", dict()).copy()
         cluster.update(self.cluster_config.get(job.name, dict()))
@@ -928,15 +976,15 @@ class ClusterExecutor(RealExecutor):
 
         return cluster
 
-    def cluster_wildcards(self, job):
+    def cluster_wildcards(self, job: ExecutorJobInterface):
         return Wildcards(fromdict=self.cluster_params(job))
 
-    def handle_job_success(self, job):
+    def handle_job_success(self, job: ExecutorJobInterface):
         super().handle_job_success(
             job, upload_remote=False, handle_log=False, handle_touch=False
         )
 
-    def handle_job_error(self, job):
+    def handle_job_error(self, job: ExecutorJobInterface):
         # TODO what about removing empty remote dirs?? This cannot be decided
         # on the cluster node.
         super().handle_job_error(job, upload_remote=False)
@@ -955,9 +1003,9 @@ class ClusterExecutor(RealExecutor):
     def print_cluster_job_error(self, job_info, jobid):
         job = job_info.job
         kind = (
-            "rule {}".format(job.rule.name)
+            f"rule {job.rule.name}"
             if not job.is_group()
-            else "group job {}".format(job.groupid)
+            else f"group job {job.groupid}"
         )
         logger.error(
             "Error executing {} on cluster (jobid: {}, external: "
@@ -977,8 +1025,8 @@ GenericClusterJob = namedtuple(
 class GenericClusterExecutor(ClusterExecutor):
     def __init__(
         self,
-        workflow,
-        dag,
+        workflow: WorkflowExecutorInterface,
+        dag: DAGExecutorInterface,
         cores,
         submitcmd="qsub",
         statuscmd=None,
@@ -995,7 +1043,6 @@ class GenericClusterExecutor(ClusterExecutor):
         max_status_checks_per_second=1,
         keepincomplete=False,
     ):
-
         self.submitcmd = submitcmd
         if not assume_shared_fs and statuscmd is None:
             raise WorkflowError(
@@ -1037,14 +1084,13 @@ class GenericClusterExecutor(ClusterExecutor):
                 "specify a cluster status command."
             )
 
-    def get_job_exec_prefix(self, job):
+    def get_job_exec_prefix(self, job: ExecutorJobInterface):
         if self.assume_shared_fs:
-            # quoting the workdir since it may contain spaces
-            return f"cd {repr(self.workflow.workdir_init)}"
+            return f"cd {shlex.quote(self.workflow.workdir_init)}"
         else:
             return ""
 
-    def get_job_exec_suffix(self, job):
+    def get_job_exec_suffix(self, job: ExecutorJobInterface):
         if self.statuscmd:
             return "exit 0 || exit 1"
         elif self.assume_shared_fs:
@@ -1057,11 +1103,11 @@ class GenericClusterExecutor(ClusterExecutor):
             )
         assert False, "bug: neither statuscmd defined nor shared FS"
 
-    def get_jobfinished_marker(self, job):
-        return os.path.join(self.tmpdir, "{}.jobfinished".format(job.jobid))
+    def get_jobfinished_marker(self, job: ExecutorJobInterface):
+        return os.path.join(self.tmpdir, f"{job.jobid}.jobfinished")
 
-    def get_jobfailed_marker(self, job):
-        return os.path.join(self.tmpdir, "{}.jobfailed".format(job.jobid))
+    def get_jobfailed_marker(self, job: ExecutorJobInterface):
+        return os.path.join(self.tmpdir, f"{job.jobid}.jobfailed")
 
     def _launch_sidecar(self):
         def copy_stdout(executor, process):
@@ -1143,12 +1189,18 @@ class GenericClusterExecutor(ClusterExecutor):
             )
             self.shutdown()
 
-    def register_job(self, job):
+    def register_job(self, job: ExecutorJobInterface):
         # Do not register job here.
         # Instead do it manually once the jobid is known.
         pass
 
-    def run(self, job, callback=None, submit_callback=None, error_callback=None):
+    def run(
+        self,
+        job: ExecutorJobInterface,
+        callback=None,
+        submit_callback=None,
+        error_callback=None,
+    ):
         super()._run(job)
         jobid = job.jobid
 
@@ -1360,8 +1412,8 @@ class SynchronousClusterExecutor(ClusterExecutor):
 
     def __init__(
         self,
-        workflow,
-        dag,
+        workflow: WorkflowExecutorInterface,
+        dag: DAGExecutorInterface,
         cores,
         submitcmd="qsub",
         cluster_config=None,
@@ -1392,7 +1444,7 @@ class SynchronousClusterExecutor(ClusterExecutor):
 
     def get_job_exec_prefix(self, job):
         if self.assume_shared_fs:
-            return f"cd {self.workflow.workdir_init}"
+            return f"cd {shlex.quote(self.workflow.workdir_init)}"
         else:
             return ""
 
@@ -1400,7 +1452,13 @@ class SynchronousClusterExecutor(ClusterExecutor):
         logger.info("Will exit after finishing currently running jobs.")
         self.shutdown()
 
-    def run(self, job, callback=None, submit_callback=None, error_callback=None):
+    def run(
+        self,
+        job: ExecutorJobInterface,
+        callback=None,
+        submit_callback=None,
+        error_callback=None,
+    ):
         super()._run(job)
 
         jobscript = self.get_jobscript(job)
@@ -1470,8 +1528,8 @@ DRMAAClusterJob = namedtuple(
 class DRMAAExecutor(ClusterExecutor):
     def __init__(
         self,
-        workflow,
-        dag,
+        workflow: WorkflowExecutorInterface,
+        dag: DAGExecutorInterface,
         cores,
         jobname="snakejob.{rulename}.{jobid}.sh",
         printreason=False,
@@ -1507,16 +1565,16 @@ class DRMAAExecutor(ClusterExecutor):
                 "Please install it, e.g. with easy_install3 --user drmaa"
             )
         except RuntimeError as e:
-            raise WorkflowError("Error loading drmaa support:\n{}".format(e))
+            raise WorkflowError(f"Error loading drmaa support:\n{e}")
         self.session = drmaa.Session()
         self.drmaa_args = drmaa_args
         self.drmaa_log_dir = drmaa_log_dir
         self.session.initialize()
         self.submitted = list()
 
-    def get_job_exec_prefix(self, job):
+    def get_job_exec_prefix(self, job: ExecutorJobInterface):
         if self.assume_shared_fs:
-            return f"cd {self.workflow.workdir_init}"
+            return f"cd {shlex.quote(self.workflow.workdir_init)}"
         else:
             return ""
 
@@ -1532,7 +1590,13 @@ class DRMAAExecutor(ClusterExecutor):
                 pass
         self.shutdown()
 
-    def run(self, job, callback=None, submit_callback=None, error_callback=None):
+    def run(
+        self,
+        job: ExecutorJobInterface,
+        callback=None,
+        submit_callback=None,
+        error_callback=None,
+    ):
         super()._run(job)
         jobscript = self.get_jobscript(job)
         self.write_jobscript(job, jobscript)
@@ -1564,14 +1628,10 @@ class DRMAAExecutor(ClusterExecutor):
             drmaa.InternalException,
             drmaa.InvalidAttributeValueException,
         ) as e:
-            print_exception(
-                WorkflowError("DRMAA Error: {}".format(e)), self.workflow.linemaps
-            )
+            print_exception(WorkflowError(f"DRMAA Error: {e}"), self.workflow.linemaps)
             error_callback(job)
             return
-        logger.info(
-            "Submitted DRMAA job {} with external jobid {}.".format(job.jobid, jobid)
-        )
+        logger.info(f"Submitted DRMAA job {job.jobid} with external jobid {jobid}.")
         self.submitted.append(jobid)
         self.session.deleteJobTemplate(jt)
 
@@ -1608,7 +1668,7 @@ class DRMAAExecutor(ClusterExecutor):
                         continue
                     except (drmaa.InternalException, Exception) as e:
                         print_exception(
-                            WorkflowError("DRMAA Error: {}".format(e)),
+                            WorkflowError(f"DRMAA Error: {e}"),
                             self.workflow.linemaps,
                         )
                         os.remove(active_job.jobscript)
@@ -1659,7 +1719,7 @@ def change_working_directory(directory=None):
     if directory:
         try:
             saved_directory = os.getcwd()
-            logger.info("Changing to shadow directory: {}".format(directory))
+            logger.info(f"Changing to shadow directory: {directory}")
             os.chdir(directory)
             yield
         finally:
@@ -1676,11 +1736,12 @@ KubernetesJob = namedtuple(
 class KubernetesExecutor(ClusterExecutor):
     def __init__(
         self,
-        workflow,
-        dag,
+        workflow: WorkflowExecutorInterface,
+        dag: DAGExecutorInterface,
         namespace,
         container_image=None,
         k8s_cpu_scalar=1.0,
+        k8s_service_account_name=None,
         jobname="{rulename}.{jobid}",
         printreason=False,
         quiet=False,
@@ -1722,6 +1783,7 @@ class KubernetesExecutor(ClusterExecutor):
         import kubernetes.client
 
         self.k8s_cpu_scalar = k8s_cpu_scalar
+        self.k8s_service_account_name = k8s_service_account_name
         self.kubeapi = kubernetes.client.CoreV1Api()
         self.batchapi = kubernetes.client.BatchV1Api()
         self.namespace = namespace
@@ -1733,7 +1795,7 @@ class KubernetesExecutor(ClusterExecutor):
         self.container_image = container_image or get_container_image()
         logger.info(f"Using {self.container_image} for Kubernetes jobs.")
 
-    def get_job_exec_prefix(self, job):
+    def get_job_exec_prefix(self, job: ExecutorJobInterface):
         return "cp -rf /source/. ."
 
     def register_secret(self):
@@ -1767,7 +1829,7 @@ class KubernetesExecutor(ClusterExecutor):
                 continue
 
             with open(f, "br") as content:
-                key = "f{}".format(i)
+                key = f"f{i}"
 
                 # Some files are smaller than 1MB, but grows larger after being base64 encoded
                 # We should exclude them as well, otherwise Kubernetes APIs will complain
@@ -1780,7 +1842,6 @@ class KubernetesExecutor(ClusterExecutor):
                         "the maximum file size (1MB) that can be passed "
                         "from host to kubernetes.".format(
                             f=f,
-                            source_file_size=source_file_size,
                             key=key,
                             encoded_size=encoded_size,
                         )
@@ -1818,7 +1879,7 @@ class KubernetesExecutor(ClusterExecutor):
                 if k in self.secret_files
             }
             for k, v in sorted(entry_sizes.items(), key=lambda item: item[1])[:-6:-1]:
-                logger.warning("  * File: {k}, original size: {v}".format(k=k, v=v))
+                logger.warning(f"  * File: {k}, original size: {v}")
 
             raise WorkflowError("ConfigMap too large")
 
@@ -1865,7 +1926,13 @@ class KubernetesExecutor(ClusterExecutor):
 
         self.shutdown()
 
-    def run(self, job, callback=None, submit_callback=None, error_callback=None):
+    def run(
+        self,
+        job: ExecutorJobInterface,
+        callback=None,
+        submit_callback=None,
+        error_callback=None,
+    ):
         import kubernetes.client
 
         super()._run(job)
@@ -1874,7 +1941,7 @@ class KubernetesExecutor(ClusterExecutor):
         # Kubernetes silently does not submit a job if the name is too long
         # therefore, we ensure that it is not longer than snakejob+uuid.
         jobid = "snakejob-{}".format(
-            get_uuid("{}-{}-{}".format(self.run_namespace, job.jobid, job.attempt))
+            get_uuid(f"{self.run_namespace}-{job.jobid}-{job.attempt}")
         )
 
         body = kubernetes.client.V1Pod()
@@ -1903,6 +1970,10 @@ class KubernetesExecutor(ClusterExecutor):
         body.spec = kubernetes.client.V1PodSpec(
             containers=[container], node_selector=node_selector
         )
+        # Add service account name if provided
+        if self.k8s_service_account_name:
+            body.spec.service_account_name = self.k8s_service_account_name
+
         # fail on first error
         body.spec.restart_policy = "Never"
 
@@ -2072,7 +2143,7 @@ class KubernetesExecutor(ClusterExecutor):
                 still_running = list()
             for j in active_jobs:
                 async with self.status_rate_limiter:
-                    logger.debug("Checking status for pod {}".format(j.jobid))
+                    logger.debug(f"Checking status for pod {j.jobid}")
                     job_not_found = False
                     try:
                         res = self._kubernetes_retry(
@@ -2133,8 +2204,8 @@ TibannaJob = namedtuple(
 class TibannaExecutor(ClusterExecutor):
     def __init__(
         self,
-        workflow,
-        dag,
+        workflow: WorkflowExecutorInterface,
+        dag: DAGExecutorInterface,
         cores,
         tibanna_sfn,
         precommand="",
@@ -2152,7 +2223,7 @@ class TibannaExecutor(ClusterExecutor):
         self.workflow_sources = []
         for wfs in dag.get_sources():
             if os.path.isdir(wfs):
-                for (dirpath, dirnames, filenames) in os.walk(wfs):
+                for dirpath, dirnames, filenames in os.walk(wfs):
                     self.workflow_sources.extend(
                         [os.path.join(dirpath, f) for f in filenames]
                     )
@@ -2174,7 +2245,7 @@ class TibannaExecutor(ClusterExecutor):
             self.precommand = ""
         self.s3_bucket = workflow.default_remote_prefix.split("/")[0]
         self.s3_subdir = re.sub(
-            "^{}/".format(self.s3_bucket), "", workflow.default_remote_prefix
+            f"^{self.s3_bucket}/", "", workflow.default_remote_prefix
         )
         logger.debug("precommand= " + self.precommand)
         logger.debug("bucket=" + self.s3_bucket)
@@ -2209,7 +2280,7 @@ class TibannaExecutor(ClusterExecutor):
         from tibanna.core import API
 
         for j in self.active_jobs:
-            logger.info("killing job {}".format(j.jobname))
+            logger.info(f"killing job {j.jobname}")
             while True:
                 try:
                     res = API().kill(j.exec_arn)
@@ -2225,7 +2296,7 @@ class TibannaExecutor(ClusterExecutor):
         if checkdir:
             checkdir = checkdir.rstrip("/")
             if f.startswith(checkdir):
-                fname = re.sub("^{}/".format(checkdir), "", f)
+                fname = re.sub(f"^{checkdir}/", "", f)
                 fdir = checkdir
             else:
                 direrrmsg = (
@@ -2239,12 +2310,12 @@ class TibannaExecutor(ClusterExecutor):
         return fname, fdir
 
     def remove_prefix(self, s):
-        return re.sub("^{}/{}/".format(self.s3_bucket, self.s3_subdir), "", s)
+        return re.sub(f"^{self.s3_bucket}/{self.s3_subdir}/", "", s)
 
     def get_snakefile(self):
         return os.path.basename(self.snakefile)
 
-    def add_command(self, job, tibanna_args, tibanna_config):
+    def add_command(self, job: ExecutorJobInterface, tibanna_args, tibanna_config):
         # format command
         command = self.format_job_exec(job)
 
@@ -2253,7 +2324,7 @@ class TibannaExecutor(ClusterExecutor):
         logger.debug("command = " + str(command))
         tibanna_args.command = command
 
-    def add_workflow_files(self, job, tibanna_args):
+    def add_workflow_files(self, job: ExecutorJobInterface, tibanna_args):
         snakefile_fname, snakemake_dir = self.split_filename(self.snakefile)
         snakemake_child_fnames = []
         for src in self.workflow_sources:
@@ -2261,6 +2332,7 @@ class TibannaExecutor(ClusterExecutor):
             if src_fname != snakefile_fname:  # redundant
                 snakemake_child_fnames.append(src_fname)
         # change path for config files
+        # TODO - this is a hacky way to do this
         self.workflow.overwrite_configfiles = [
             self.split_filename(cf, snakemake_dir)[0]
             for cf in self.workflow.overwrite_configfiles
@@ -2280,7 +2352,7 @@ class TibannaExecutor(ClusterExecutor):
             rel = f
         return rel
 
-    def make_tibanna_input(self, job):
+    def make_tibanna_input(self, job: ExecutorJobInterface):
         from tibanna import ec2_utils, core as tibanna_core
 
         # input & output
@@ -2339,9 +2411,9 @@ class TibannaExecutor(ClusterExecutor):
         # jobid, grouping, run_name
         jobid = tibanna_core.create_jobid()
         if job.is_group():
-            run_name = "snakemake-job-%s-group-%s" % (str(jobid), str(job.groupid))
+            run_name = f"snakemake-job-{str(jobid)}-group-{str(job.groupid)}"
         else:
-            run_name = "snakemake-job-%s-rule-%s" % (str(jobid), str(job.rule))
+            run_name = f"snakemake-job-{str(jobid)}-rule-{str(job.rule)}"
 
         # tibanna input
         tibanna_config = {
@@ -2372,7 +2444,13 @@ class TibannaExecutor(ClusterExecutor):
         logger.debug(json.dumps(tibanna_input, indent=4))
         return tibanna_input
 
-    def run(self, job, callback=None, submit_callback=None, error_callback=None):
+    def run(
+        self,
+        job: ExecutorJobInterface,
+        callback=None,
+        submit_callback=None,
+        error_callback=None,
+    ):
         logger.info("running job using Tibanna...")
         from tibanna.core import API
 
@@ -2422,7 +2500,7 @@ class TibannaExecutor(ClusterExecutor):
                     else:
                         status = "FAILED_AT_SUBMISSION"
                     if not self.quiet or status != "RUNNING":
-                        logger.debug("job %s: %s" % (j.jobname, status))
+                        logger.debug(f"job {j.jobname}: {status}")
                     if status == "RUNNING":
                         still_running.append(j)
                     elif status == "SUCCEEDED":
