@@ -3,26 +3,28 @@ __copyright__ = "Copyright 2022, Johannes Köster"
 __email__ = "johannes.koester@uni-due.de"
 __license__ = "MIT"
 
-from abc import abstractmethod
-import enum
 import os
 import sys
 import base64
 import tempfile
 import json
 import shutil
-import copy
 
 from collections import defaultdict
 from itertools import chain, filterfalse
 from operator import attrgetter
 from typing import Optional
+from abc import ABC, abstractmethod
+from snakemake.interfaces import (
+    ExecutorJobInterface,
+    GroupJobExecutorInterface,
+    SingleJobExecutorInterface,
+)
 
 from snakemake.io import (
     IOFile,
     Wildcards,
     Resources,
-    _IOFile,
     is_flagged,
     get_flag_value,
     wait_for_files,
@@ -36,10 +38,10 @@ from snakemake.logging import logger
 from snakemake.common import (
     DYNAMIC_FILL,
     is_local_file,
-    parse_uri,
     lazy_property,
     get_uuid,
     TBDString,
+    IO_PROP_LIMIT,
 )
 
 
@@ -64,39 +66,21 @@ def jobfiles(jobs, type):
     return chain(*map(attrgetter(type), jobs))
 
 
-class AbstractJob:
-    def logfile_suggestion(self, prefix: str) -> str:
-        raise NotImplementedError()
-
-    def is_group(self):
-        raise NotImplementedError()
-
-    def log_info(self, skip_dynamic=False):
-        raise NotImplementedError()
-
-    def log_error(self, msg=None, **kwargs):
-        raise NotImplementedError()
-
-    def remove_existing_output(self):
-        raise NotImplementedError()
-
-    def download_remote_input(self):
-        raise NotImplementedError()
-
-    def properties(self, omit_resources=["_cores", "_nodes"], **aux_properties):
-        raise NotImplementedError()
-
+class AbstractJob(ExecutorJobInterface):
+    @abstractmethod
     def reset_params_and_resources(self):
-        raise NotImplementedError()
+        ...
 
+    @abstractmethod
     def get_target_spec(self):
-        raise NotImplementedError()
+        ...
 
-    def products(self):
-        raise NotImplementedError()
+    @abstractmethod
+    def products(self, include_logfiles=True):
+        ...
 
-    def has_products(self):
-        for o in self.products():
+    def has_products(self, include_logfiles=True):
+        for o in self.products(include_logfiles=include_logfiles):
             return True
         return False
 
@@ -156,7 +140,7 @@ class JobFactory:
         return obj
 
 
-class Job(AbstractJob):
+class Job(AbstractJob, SingleJobExecutorInterface):
     HIGHEST_PRIORITY = sys.maxsize
 
     obj_cache = dict()
@@ -165,18 +149,18 @@ class Job(AbstractJob):
         "rule",
         "dag",
         "wildcards_dict",
-        "wildcards",
+        "_wildcards",
         "_format_wildcards",
-        "input",
+        "_input",
         "dependencies",
-        "output",
+        "_output",
         "_params",
         "_log",
         "_benchmark",
         "_resources",
         "_conda_env_file",
         "_conda_env",
-        "shadow_dir",
+        "_shadow_dir",
         "_inputsize",
         "dynamic_output",
         "dynamic_input",
@@ -223,10 +207,7 @@ class Job(AbstractJob):
             input_mapping,
             self.dependencies,
             self.incomplete_input_expand,
-        ) = self.rule.expand_input(
-            self.wildcards_dict,
-            groupid=groupid,
-        )
+        ) = self.rule.expand_input(self.wildcards_dict, groupid=groupid)
 
         self.output, output_mapping = self.rule.expand_output(self.wildcards_dict)
         # other properties are lazy to be able to use additional parameters and check already existing files
@@ -244,7 +225,7 @@ class Job(AbstractJob):
 
         self.shadow_dir = None
         self._inputsize = None
-        self.is_updated = False
+        self._is_updated = False
         self._params_and_resources_resetted = False
 
         self._attempt = self.dag.workflow.attempt
@@ -283,6 +264,46 @@ class Job(AbstractJob):
                         )
                 self.subworkflow_input[f] = sub
 
+    @property
+    def is_updated(self):
+        return self._is_updated
+
+    @is_updated.setter
+    def is_updated(self, value):
+        self._is_updated = value
+
+    @property
+    def shadow_dir(self):
+        return self._shadow_dir
+
+    @shadow_dir.setter
+    def shadow_dir(self, value):
+        self._shadow_dir = value
+
+    @property
+    def wildcards(self):
+        return self._wildcards
+
+    @wildcards.setter
+    def wildcards(self, value):
+        self._wildcards = value
+
+    @property
+    def input(self):
+        return self._input
+
+    @input.setter
+    def input(self, value):
+        self._input = value
+
+    @property
+    def output(self):
+        return self._output
+
+    @output.setter
+    def output(self, value):
+        self._output = value
+
     def logfile_suggestion(self, prefix: str) -> str:
         """Return a suggestion for the log file name given a prefix."""
         return (
@@ -351,7 +372,7 @@ class Job(AbstractJob):
     def params(self):
         if self._params is None:
             self._params = self.rule.expand_params(
-                self.wildcards_dict, self.input, self.output, self.resources
+                self.wildcards_dict, self.input, self.output, self
             )
         return self._params
 
@@ -509,7 +530,7 @@ class Job(AbstractJob):
             raise RuleException(str(ex), rule=self.rule)
         except KeyError as ex:
             raise RuleException(
-                "Unknown variable in message " "of shell command: {}".format(str(ex)),
+                "Unknown variable in message of shell command: {}".format(str(ex)),
                 rule=self.rule,
             )
 
@@ -526,7 +547,7 @@ class Job(AbstractJob):
             raise RuleException(str(ex), rule=self.rule)
         except KeyError as ex:
             raise RuleException(
-                "Unknown variable when printing " "shell command: {}".format(str(ex)),
+                "Unknown variable when printing shell command: {}".format(str(ex)),
                 rule=self.rule,
             )
 
@@ -681,7 +702,7 @@ class Job(AbstractJob):
                 if f in requested:
                     if f in self.dynamic_output:
                         if not self.expand_dynamic(f_):
-                            yield "{} (dynamic)".format(f_)
+                            yield f"{f_} (dynamic)"
                     else:
                         yield from handle_file(f)
         else:
@@ -846,8 +867,13 @@ class Job(AbstractJob):
         if self.benchmark:
             self.benchmark.prepare()
 
-        # wait for input files
-        wait_for_files(self.input, latency_wait=self.dag.workflow.latency_wait)
+        # wait for input files, respecting keep_remote_local
+        force_stay_on_remote = not self.dag.keep_remote_local
+        wait_for_files(
+            self.input,
+            force_stay_on_remote=force_stay_on_remote,
+            latency_wait=self.dag.workflow.latency_wait,
+        )
 
         if not self.is_shadow:
             return
@@ -865,7 +891,7 @@ class Job(AbstractJob):
             or self.rule.shadow_depth == "copy-minimal"
         ):
             # Re-create the directory structure in the shadow directory
-            for (f, d) in set(
+            for f, d in set(
                 [
                     (item, os.path.dirname(item))
                     for sublist in [self.input, self.output, self.log]
@@ -999,8 +1025,8 @@ class Job(AbstractJob):
             "type": "single",
             "rule": self.rule.name,
             "local": self.is_local,
-            "input": self.input,
-            "output": self.output,
+            "input": None if len(self.input) > IO_PROP_LIMIT else self.input,
+            "output": None if len(self.output) > IO_PROP_LIMIT else self.output,
             "wildcards": self.wildcards_dict,
             "params": params,
             "log": self.log,
@@ -1225,13 +1251,12 @@ class GroupJobFactory:
         return obj
 
 
-class GroupJob(AbstractJob):
-
+class GroupJob(AbstractJob, GroupJobExecutorInterface):
     obj_cache = dict()
 
     __slots__ = [
-        "groupid",
-        "jobs",
+        "_groupid",
+        "_jobs",
         "_resources",
         "_input",
         "_output",
@@ -1239,15 +1264,15 @@ class GroupJob(AbstractJob):
         "_inputsize",
         "_all_products",
         "_attempt",
-        "toposorted",
+        "_toposorted",
         "_jobid",
     ]
 
     def __init__(self, id, jobs, global_resources):
         self.groupid = id
-        self.jobs = jobs
+        self._jobs = jobs
         self.global_resources = global_resources
-        self.toposorted = None
+        self._toposorted = None
         self._resources = None
         self._scheduler_resources = None
         self._input = None
@@ -1258,6 +1283,30 @@ class GroupJob(AbstractJob):
         self._attempt = self.dag.workflow.attempt
         self._jobid = None
 
+    @property
+    def groupid(self):
+        return self._groupid
+
+    @groupid.setter
+    def groupid(self, new_groupid):
+        self._groupid = new_groupid
+
+    @property
+    def jobs(self):
+        return self._jobs
+
+    @jobs.setter
+    def jobs(self, new_jobs):
+        self._jobs = new_jobs
+
+    @property
+    def toposorted(self):
+        return self._toposorted
+
+    @toposorted.setter
+    def toposorted(self, new_toposorted):
+        self._toposorted = new_toposorted
+
     def logfile_suggestion(self, prefix: str) -> str:
         """Return a suggestion for the log file name given a prefix."""
         return f"{prefix}/groupjobs/group_{self.name}/job_{self.jobid}.log"
@@ -1272,7 +1321,6 @@ class GroupJob(AbstractJob):
 
     def finalize(self):
         if self.toposorted is None:
-
             self.toposorted = [
                 *self.dag.toposorted(self.jobs, inherit_pipe_dependencies=True)
             ]
@@ -1286,7 +1334,7 @@ class GroupJob(AbstractJob):
             yield from layer
 
     def __repr__(self):
-        return "JobGroup({},{})".format(self.groupid, repr(self.jobs))
+        return f"JobGroup({self.groupid},{repr(self.jobs)})"
 
     def __contains__(self, job):
         return job in self.jobs
@@ -1434,8 +1482,8 @@ class GroupJob(AbstractJob):
             "type": "group",
             "groupid": self.groupid,
             "local": self.is_local,
-            "input": self.input,
-            "output": self.output,
+            "input": None if len(self.input) > IO_PROP_LIMIT else self.input,
+            "output": None if len(self.output) > IO_PROP_LIMIT else self.output,
             "threads": self.threads,
             "resources": resources,
             "jobid": self.jobid,
@@ -1539,13 +1587,9 @@ class GroupJob(AbstractJob):
         try:
             return format(string, **_variables)
         except NameError as ex:
-            raise WorkflowError(
-                "NameError with group job {}: {}".format(self.jobid, str(ex))
-            )
+            raise WorkflowError(f"NameError with group job {self.jobid}: {str(ex)}")
         except IndexError as ex:
-            raise WorkflowError(
-                "IndexError with group job {}: {}".format(self.jobid, str(ex))
-            )
+            raise WorkflowError(f"IndexError with group job {self.jobid}: {str(ex)}")
         except Exception as ex:
             raise WorkflowError(
                 f"Error when formatting {string} for group job {self.jobid}: {ex}"
@@ -1608,7 +1652,6 @@ class GroupJob(AbstractJob):
 
 
 class Reason:
-
     __slots__ = [
         "_updated_input",
         "_updated_input_run",
