@@ -6,12 +6,11 @@ __license__ = "MIT"
 from pathlib import Path
 import types
 import re
+from snakemake.common import Rules
 
 from snakemake.exceptions import WorkflowError
 from snakemake.path_modifier import PathModifier
 from snakemake import wrapper
-from snakemake.checkpoints import Checkpoints
-from snakemake.common import Rules, Scatter, Gather
 
 
 def get_name_modifier_func(rules=None, name_modifier=None, parent_modifier=None):
@@ -52,6 +51,8 @@ class ModuleInfo:
         self.meta_wrapper = meta_wrapper
         self.config = config
         self.skip_validation = skip_validation
+        self.parent_modifier = self.workflow.modifier
+        self.rule_proxies = Rules()
 
         if prefix is not None:
             if isinstance(prefix, Path):
@@ -78,7 +79,7 @@ class ModuleInfo:
         skip_global_report_caption=False,
     ):
         snakefile = self.get_snakefile()
-        with WorkflowModifier(
+        modifier = WorkflowModifier(
             self.workflow,
             config=self.config,
             base_snakefile=snakefile,
@@ -87,15 +88,21 @@ class ModuleInfo:
             skip_global_report_caption=skip_global_report_caption,
             rule_exclude_list=exclude_rules,
             rule_whitelist=self.get_rule_whitelist(rules),
-            rulename_modifier=get_name_modifier_func(rules, name_modifier),
+            resolved_rulename_modifier=get_name_modifier_func(
+                rules, name_modifier, parent_modifier=self.parent_modifier
+            ),
+            local_rulename_modifier=get_name_modifier_func(rules, name_modifier),
             ruleinfo_overwrite=ruleinfo,
             allow_rule_overwrite=True,
             namespace=self.name,
             replace_prefix=self.replace_prefix,
             prefix=self.prefix,
             replace_wrapper_tag=self.get_wrapper_tag(),
-        ):
+            rule_proxies=self.rule_proxies,
+        )
+        with modifier:
             self.workflow.include(snakefile, overwrite_default_target=True)
+            self.parent_modifier.inherit_rule_proxies(modifier)
 
     def get_snakefile(self):
         if self.meta_wrapper:
@@ -140,7 +147,8 @@ class WorkflowModifier:
         skip_configfile=False,
         skip_validation=False,
         skip_global_report_caption=False,
-        rulename_modifier=None,
+        resolved_rulename_modifier=None,
+        local_rulename_modifier=None,
         rule_whitelist=None,
         rule_exclude_list=None,
         ruleinfo_overwrite=None,
@@ -149,13 +157,15 @@ class WorkflowModifier:
         prefix=None,
         replace_wrapper_tag=None,
         namespace=None,
+        rule_proxies=None,
     ):
         if parent_modifier is not None:
             # init with values from parent modifier
             self.base_snakefile = parent_modifier.base_snakefile
             self.globals = parent_modifier.globals
             self.skip_configfile = parent_modifier.skip_configfile
-            self.rulename_modifier = parent_modifier.rulename_modifier
+            self.resolved_rulename_modifier = parent_modifier.resolved_rulename_modifier
+            self.local_rulename_modifier = parent_modifier.local_rulename_modifier
             self.skip_validation = parent_modifier.skip_validation
             self.skip_global_report_caption = parent_modifier.skip_global_report_caption
             self.rule_whitelist = parent_modifier.rule_whitelist
@@ -165,11 +175,18 @@ class WorkflowModifier:
             self.path_modifier = parent_modifier.path_modifier
             self.replace_wrapper_tag = parent_modifier.replace_wrapper_tag
             self.namespace = parent_modifier.namespace
+            self.wildcard_constraints = parent_modifier.wildcard_constraints
+            self.rules = parent_modifier.rules
+            self.rule_proxies = parent_modifier.rule_proxies
         else:
             # default settings for globals if not inheriting from parent
             self.globals = (
                 globals if globals is not None else dict(workflow.vanilla_globals)
             )
+            self.wildcard_constraints = dict()
+            self.rules = set()
+            self.rule_proxies = rule_proxies or Rules()
+            self.globals["rules"] = self.rule_proxies
 
         self.workflow = workflow
         self.base_snakefile = base_snakefile
@@ -178,7 +195,8 @@ class WorkflowModifier:
             self.globals["config"] = config
 
         self.skip_configfile = skip_configfile
-        self.rulename_modifier = rulename_modifier
+        self.resolved_rulename_modifier = resolved_rulename_modifier
+        self.local_rulename_modifier = local_rulename_modifier
         self.skip_validation = skip_validation
         self.skip_global_report_caption = skip_global_report_caption
         self.rule_whitelist = rule_whitelist
@@ -189,14 +207,20 @@ class WorkflowModifier:
         self.replace_wrapper_tag = replace_wrapper_tag
         self.namespace = namespace
 
+    def inherit_rule_proxies(self, child_modifier):
+        for name, rule in child_modifier.rule_proxies._rules.items():
+            if child_modifier.local_rulename_modifier is not None:
+                name = child_modifier.local_rulename_modifier(name)
+            self.rule_proxies._register_rule(name, rule)
+
     def skip_rule(self, rulename):
         return (
             self.rule_whitelist is not None and rulename not in self.rule_whitelist
         ) or (self.rule_exclude_list is not None and rulename in self.rule_exclude_list)
 
     def modify_rulename(self, rulename):
-        if self.rulename_modifier is not None:
-            return self.rulename_modifier(rulename)
+        if self.resolved_rulename_modifier is not None:
+            return self.resolved_rulename_modifier(rulename)
         return rulename
 
     def modify_path(self, path, property=None):
