@@ -6,15 +6,12 @@ __license__ = "MIT"
 import base64
 import os
 import re
-import struct
 import time
 
 from snakemake.remote import AbstractRemoteObject, AbstractRemoteProvider
 from snakemake.exceptions import WorkflowError, CheckSumMismatchException
 from snakemake.common import lazy_property
 import snakemake.io
-from snakemake.utils import os_sync
-from snakemake.logging import logger
 
 try:
     import google.cloud
@@ -113,7 +110,6 @@ class Crc32cCalculator:
 
 
 class RemoteProvider(AbstractRemoteProvider):
-
     supports_default = True
 
     def __init__(
@@ -183,7 +179,7 @@ class RemoteObject(AbstractRemoteObject):
         subfolder = os.path.dirname(self.blob.name)
         for blob in self.client.list_blobs(self.bucket_name, prefix=subfolder):
             # By way of being listed, it exists. mtime is a datetime object
-            name = "{}/{}".format(blob.bucket.name, blob.name)
+            name = f"{blob.bucket.name}/{blob.name}"
             cache.exists_remote[name] = True
             cache.mtime[name] = snakemake.io.Mtime(remote=blob.updated.timestamp())
             cache.size[name] = blob.size
@@ -193,12 +189,12 @@ class RemoteObject(AbstractRemoteObject):
 
         # Mark bucket and prefix as having an inventory, such that this method is
         # only called once for the subfolder in the bucket.
-        cache.exists_remote.has_inventory.add("%s/%s" % (self.bucket_name, subfolder))
+        cache.exists_remote.has_inventory.add(f"{self.bucket_name}/{subfolder}")
 
     # === Implementations of abstract class members ===
 
     def get_inventory_parent(self):
-        return self.bucket_name
+        return f"{self.bucket_name}/{os.path.dirname(self.blob.name)}"
 
     @retry.Retry(predicate=google_cloud_retry_predicate)
     def exists(self):
@@ -266,7 +262,7 @@ class RemoteObject(AbstractRemoteObject):
         os.makedirs(self.local_file(), exist_ok=True)
 
         for blob in self.directory_entries():
-            local_name = "{}/{}".format(blob.bucket.name, blob.name)
+            local_name = f"{blob.bucket.name}/{blob.name}"
 
             # Don't try to create "directory blob"
             if os.path.exists(local_name) and os.path.isdir(local_name):
@@ -287,7 +283,6 @@ class RemoteObject(AbstractRemoteObject):
             # Distinguish between single file, and folder
             f = self.local_file()
             if os.path.isdir(f):
-
                 # Ensure the "directory" exists
                 self.blob.upload_from_string(
                     "", content_type="application/x-www-form-urlencoded;charset=UTF-8"
