@@ -21,9 +21,7 @@ except ImportError as e:
 
 
 class RemoteProvider(AbstractRemoteProvider):
-
     supports_default = True
-    allows_directories = True
 
     def __init__(
         self,
@@ -65,11 +63,7 @@ class RemoteObject(PooledDomainObject):
 
     def get_default_kwargs(self, **defaults):
         """define defaults beyond those set in PooledDomainObject"""
-        return super().get_default_kwargs(
-            **{
-                "port": 22,
-            }
-        )
+        return super().get_default_kwargs(**{"port": 22})
 
     def create_connection(self, *args_to_use, **kwargs_to_use):
         """open an SFTP connection"""
