@@ -4,16 +4,13 @@ __email__ = "oliver.stolpe@bihealth.org"
 __license__ = "MIT"
 
 import os
-import re
 
-from contextlib import contextmanager
-from datetime import datetime, timedelta
+from datetime import datetime
 from pytz import timezone
 
 # module-specific
 from snakemake.remote import (
     AbstractRemoteProvider,
-    AbstractRemoteObject,
     AbstractRemoteRetryObject,
 )
 from snakemake.exceptions import WorkflowError
@@ -66,7 +63,6 @@ def _irods_session(*args, **kwargs):
 
 
 class RemoteProvider(AbstractRemoteProvider):
-
     supports_default = True
 
     def __init__(
@@ -216,7 +212,7 @@ class RemoteObject(AbstractRemoteRetryObject):
             try:
                 self._irods_session.collections.get(collpath)
             # ignore subdirectories where user does not have access
-            except (CAT_NO_ACCESS_PERMISSION):
+            except CAT_NO_ACCESS_PERMISSION:
                 pass
             except:
                 self._irods_session.collections.create(collpath)
