@@ -1160,9 +1160,7 @@ def test_group_job_resources_with_pipe(mocker):
         dpath("test_group_with_pipe"),
         cluster="./qsub",
         cores=6,
-        resources={
-            "mem_mb": 60000,
-        },
+        resources={"mem_mb": 60000},
         group_components={0: 5},
         default_resources=DefaultResources(["mem_mb=0"]),
     )
@@ -1186,9 +1184,7 @@ def test_group_job_resources_with_pipe_with_too_much_constraint():
         dpath("test_group_with_pipe"),
         cluster="./qsub",
         cores=6,
-        resources={
-            "mem_mb": 20000,
-        },
+        resources={"mem_mb": 20000},
         group_components={0: 5},
         shouldfail=True,
         default_resources=DefaultResources(["mem_mb=0"]),
@@ -1384,6 +1380,7 @@ def test_issue1281():
     run(dpath("test_issue1281"))
 
 
+@skip_on_windows  # TODO on windows, dot command is suddenly not found anymore although it is installed
 def test_filegraph():
     workdir = dpath("test_filegraph")
     dot_path = os.path.join(workdir, "fg.dot")
@@ -1395,13 +1392,11 @@ def test_filegraph():
         dot_path = dot_path.replace("\\", "/")
 
     # make sure the calls work
-    shell("cd {workdir};python -m snakemake --filegraph > {dot_path}")
+    shell("cd {workdir}; python -m snakemake --filegraph > {dot_path}")
 
     # make sure the output can be interpreted by dot
-    with open(dot_path, "rb") as dot_file, open(pdf_path, "wb") as pdf_file:
-        pdf_file.write(
-            subprocess.check_output(["dot", "-Tpdf"], stdin=dot_file, cwd=workdir)
-        )
+    shell("cd {workdir}; dot -Tpdf > {pdf_path} < {dot_path}")
+
     # make sure the generated pdf file is not empty
     assert os.stat(pdf_path).st_size > 0
 
@@ -1500,6 +1495,7 @@ def test_output_file_cache_remote():
 
 @connected
 @zenodo
+@pytest.mark.xfail(reason="zenodo currently returns an internal server error")
 def test_remote_zenodo():
     run(dpath("test_remote_zenodo"))
 
@@ -1709,6 +1705,10 @@ def test_modules_all():
     run(dpath("test_modules_all"), targets=["a"])
 
 
+def test_module_nested():
+    run(dpath("test_module_nested"))
+
+
 def test_modules_all_exclude_1():
     # Fail due to conflicting rules
     run(dpath("test_modules_all_exclude"), shouldfail=True)
@@ -1957,6 +1957,10 @@ def test_github_issue1498():
     run(dpath("test_github_issue1498"))
 
 
+def test_lazy_resources():
+    run(dpath("test_lazy_resources"))
+
+
 def test_cleanup_metadata_fail():
     run(dpath("test09"), cleanup_metadata=["xyz"])
 
@@ -1966,6 +1970,10 @@ def test_github_issue1389():
     run(dpath("test_github_issue1389"), resources={"foo": 4}, shouldfail=True)
 
 
+def test_github_issue2142():
+    run(dpath("test_github_issue2142"))
+
+
 def test_ensure_nonempty_fail():
     run(dpath("test_ensure"), targets=["a"], shouldfail=True)
 
@@ -2019,6 +2027,15 @@ def test_conda_python_script():
     run(dpath("test_conda_python_script"), use_conda=True)
 
 
+def test_conda_python_3_7_script():
+    run(dpath("test_conda_python_3_7_script"), use_conda=True)
+
+
+def test_prebuilt_conda_script():
+    sp.run("conda env create -f tests/test_prebuilt_conda_script/env.yaml", shell=True)
+    run(dpath("test_prebuilt_conda_script"), use_conda=True)
+
+
 @skip_on_windows
 def test_github_issue1818():
     run(dpath("test_github_issue1818"), rerun_triggers="input")
@@ -2048,3 +2065,42 @@ def test_github_issue1882():
 @skip_on_windows  # not platform dependent
 def test_inferred_resources():
     run(dpath("test_inferred_resources"))
+
+
+@skip_on_windows  # not platform dependent
+def test_workflow_profile():
+    test_path = dpath("test_workflow_profile")
+    general_profile = os.path.join(test_path, "dummy-general-profile")
+    # workflow profile is loaded by default
+    run(
+        test_path,
+        snakefile="workflow/Snakefile",
+        shellcmd=f"snakemake --profile {general_profile} -c1",
+    )
+
+
+@skip_on_windows  # not platform dependent
+def test_no_workflow_profile():
+    test_path = dpath("test_no_workflow_profile")
+    general_profile = os.path.join(test_path, "dummy-general-profile")
+    # workflow profile is loaded by default
+    run(
+        test_path,
+        snakefile="workflow/Snakefile",
+        shellcmd=f"snakemake --profile {general_profile} --workflow-profile none -c1",
+    )
+
+
+@skip_on_windows
+def test_localrule():
+    run(dpath("test_localrule"), targets=["1.txt", "2.txt"])
+
+
+@skip_on_windows
+def test_module_wildcard_constraints():
+    run(dpath("test_module_wildcard_constraints"))
+
+
+@skip_on_windows
+def test_config_yte():
+    run(dpath("test_config_yte"))
